<?php /*windu.org core*/
class resourceManager
{
	public static function loadResourceByPlugins(array $plugins = null,$bucket,$type){
		$pluginsResource = pluginManager::getConfig($type,$plugins);
		return self::loadResource($pluginsResource,$bucket,$type);
	}

	private static function cacheResources(array $resourcesArray,$bucket,$type) {
		$cacheFilePath = __SITE_PATH.'/cache/resources/'.$bucket.'.'.$type;
		$cacheFilePathReturn = HOME.'cache/resources/'.$bucket.'.'.$type;
		
		if (file_exists($cacheFilePath)) {
			return array('cache.'.$bucket => $cacheFilePathReturn);
		}
		foreach ($resourcesArray as $resource){
			$path = str_replace(HOME, __SITE_PATH.'/', $resource);

			if ($type=='css'){
				$resourceString .= generator::compressCSS(baseFile::readFile($path));
			}elseif ($type=='js'){
				$resourceString .= jsmin::minify(baseFile::readFile($path)."\n");
			}else{
				$compressedResource = baseFile::readFile($path);
			}			
			
			$resourcePathPom = str_replace(__SITE_PATH.'/', '', $resource);
			$resourcePathPomArray = explode('/', $resourcePathPom);
			unset($resourcePathPomArray[count($resourcePathPomArray)-1]);
			unset($resourcePathPomArray[count($resourcePathPomArray)-1]);
			$resourcePathReplace = null;
			foreach ($resourcePathPomArray as $part) {
				$resourcePathReplace.=$part.'/';
			}
			
			$resourcePathReplace = HOME.$resourcePathReplace;	
		
			if ($type=='css') {
				$resourceString = str_replace('../', $resourcePathReplace, $resourceString);
				$resourceString = str_replace('{$HOME}/', HOME, $resourceString);
			}
		}

		baseFile::saveFile($cacheFilePath, $resourceString);
		return array('cache.'.$bucket => $cacheFilePathReturn);
	}

	public static function loadResource(array $resources, $bucket, $type) {
		foreach ($resources as $key => $resource){
			$resourcePath = str_replace('{$HOME}', __SITE_PATH.'/', $resource);
			if ( filesize($resourcePath)>3) {
				if (file_exists($resourcePath)){
					if (preg_match('/nocache.*/', $key) or config::get('cacheResources')==0) {
						$resourceListNoCache[$key] = str_replace(__SITE_PATH.'/', HOME, $resourcePath);
					}else{
						$resourceListCache[$key] =  $resourcePath;
					}	
				}
			}
		}

		if (is_array($resourceListNoCache)){ksort($resourceListNoCache,SORT_NUMERIC);}
		if (is_array($resourceListCache)){
			ksort($resourceListCache,SORT_NUMERIC);
			$resourceListCache = self::cacheResources($resourceListCache, $bucket, $type);
		}
		if (is_array($resourceListCache) and is_array($resourceListNoCache)) {
			$finalResourcesArray = array_merge($resourceListCache,$resourceListNoCache);
		}elseif (is_array($resourceListCache)){
			$finalResourcesArray = $resourceListCache;
		}else{
			$finalResourcesArray = $resourceListNoCache;
		}
		return $finalResourcesArray;
	}
	
	public static function loadFrontCSS() {
		$file = __SITE_PATH.'/cache/system/frontcsslist-'.config::get('template').'.tmp';
		if (file_exists($file)){
			$resourcesCSS = unserialize(file_get_contents($file));
		}else{
			$plugins = explode(',', FRONT_PLUGINS);
			$resourcesCSS = array_merge(
				(array)self::loadResourceByPlugins($plugins,'front-system','css'),
				(array)self::loadResource(widgetsDB::getAllResources('css'),'front-widgets', 'css'),
				(array)self::loadResource(themesDB::getAllResources('css'),'front-theme', 'css')
			);
			file_put_contents($file, serialize($resourcesCSS));				
		}
		return $resourcesCSS;
	}
	public static function loadFrontJS(){
		$file = __SITE_PATH.'/cache/system/frontjslist-'.config::get('template').'.tmp';
		if (file_exists($file)){
	 		$resourcesJS = unserialize(file_get_contents($file));
		}else{
			$plugins = explode(',', FRONT_PLUGINS);
			$resourcesJS = array_merge(
				(array)self::loadResourceByPlugins($plugins,'front-system','js'),
				(array)self::loadResource(themesDB::getAllResources('js'),'front-theme', 'js')
			);	
			if (config::get('inPlaceEditor')==1) {
				$resourcesJS = array_merge($resourcesJS,array(
					HOME.'app/plugins/html/resources/ckeditor/ckeditor.js',
					HOME.'app/plugins/html/resources/js/ckeditor.inline.config.js'
				));
			}
			file_put_contents($file, serialize($resourcesJS));			
		}
		return $resourcesJS;	
	}
	public static function loadAdminCSS(){
		$file = __SITE_PATH.'/cache/system/admincsslist.tmp';
		if (file_exists($file)){
			$admincsslist = unserialize(file_get_contents($file));
		}else{
			$plugins = explode(',', ADMIN_PLUGINS);
			$admincsslist = self::loadResourceByPlugins($plugins,'admin','css');
			file_put_contents($file, serialize($admincsslist));	
		}
		
		return $admincsslist;
	}
	public static function loadAdminJS(){
		$file = __SITE_PATH.'/cache/system/adminjslist.tmp';
		if (file_exists($file)){
			$admincsslist = unserialize(file_get_contents($file));
		}else{
			$plugins = explode(',', ADMIN_PLUGINS);
			$admincsslist = self::loadResourceByPlugins($plugins,'admin','js');
			file_put_contents($file, serialize($admincsslist));	
		}
		
		return $admincsslist;		
	}
}
?>