<?php /*windu.org database*/
/**
 * Class for executig sql queries
 * @author Krzysztof Ruszczyński <http://www.ruszczak.eu>
 * @author Adam Czajkowski
 */

class baseDB extends DB
{
	/**
	 * @var String
	 */
	public $tab;

	/**
	 * @var PDO
	 */
	protected $db;

	function __construct()
	{
		$this->tab = str_replace('db', '', strtolower(get_class($this)));
		$this->setDb( parent::getInstance() );
	}

	/**
	 * Function created in order to change database connection during tests
	 * @param mixed $db
	 */
	public function setDb( $db )
	{
		$this->db = $db;
	}

	/**
	 * @param String $sql sql query
	 * @param array $bindValues
	 *
	 * @return PDOStatement
	 */
	protected function _createStatement( $sql, $bindValues )
	{
		$stmt = $this->db->prepare( $sql );

		foreach( $bindValues as $key=>$value )
		{
			if( is_numeric($key) )
			{
				$key++;
			}
			else if( $key{0} <> ':' )
			{
				$key = ':' . $key;
			}

			$stmt->bindValue( $key, $value );
		}
		return $stmt;
	}

	/**
	 * @param string $where
	 * @param string $order
	 * @param string $what
	 * @param string $limit
	 * @param string $groupby
	 * @param array $bindValues
	 * @return PDOStatement
	 */
	public function select($where = null, $order = null, $what = '*',$limit = null, $groupby = null, $bindValues = array() )
	{
		if ($where!=null){$where='WHERE '.$where;}
		if ($order!=null){$order='ORDER BY '.$order;}
		if ($limit!=null){$limit='LIMIT '.$limit;}
		if ($groupby!=null){$groupby='GROUP BY '.$groupby;}

		if (is_array($what)) {
			$what = implode(",", $what);
		}

		$sql = "SELECT $what FROM $this->tab $where $order $limit $groupby";

		if(!empty($bindValues))
		{
			return $this->_createStatement($sql, $bindValues);
		}

		return $this->db->query($sql);
	}

	/**
	 * @param array $data
	 * @return void
	 */
	public function insert(array $data = array())
	{
		if (DB_READ_ONLY_MODE==1) {
			if (!$this->canWrite()) {return $column;}
		}	
		
		if( !is_array( current($data) ) )
		{
			return $this->_insertOneRow( $data );
		}
		else
		{
			foreach ($data as $row)
			{
				$this->_insertOneRow( $row );
			}
		}

	}

	/**
	 * @param array $row
	 * @return void
	 */
	protected function _insertOneRow( $row )
	{
		if (DB_READ_ONLY_MODE==1) {
			if (!$this->canWrite()) {return $column;}
		}		
		
		$bindValues = array();
		$keys='';
		$values='';
		foreach($row as $key=>$val)
		{
			$val = str_replace("'", "&#39;", $val);
			$keys.='`'.$key.'`,';
			$values.=":$key,";
			$bindValues[":$key"] = $val;
			//$values.="'".$val."',";
		}
		$values=rtrim($values, ',');
		$keys=rtrim($keys, ',');

		$sql = "INSERT INTO $this->tab($keys) VALUES ($values)";
		$stmt = $this->db->prepare($sql);
		$stmt->execute( $bindValues );
		return $row;
	}

	/**
	 *
	 * @param string $column
	 * @param string $value
	 * @param string $where
	 * @param array $bindValues
	 * @return void
	 */
	public function update($column, $value, $where, $bindValues = array() )
	{
		if (DB_READ_ONLY_MODE==1) {
			if (!$this->canWrite()) {return $column;}
		}		
		
		$value = str_replace("'", "&#39;", $value);
		$where='WHERE '.$where;
		$sql = "UPDATE {$this->tab} SET {$column}=:{$column} {$where}";
		$bindValues[":$column"] = $value;

		$this->_createStatement( $sql, $bindValues )->execute();
	}

	/**
	 * @param array $data
	 * @param string $where
	 * @param array $bindValues
	 * @return void
	 */
	public function updateRow($data, $where, $bindValues = array() )
	{
		if (DB_READ_ONLY_MODE==1) {
			if (!$this->canWrite()) {return $column;}
		}
		
		$sql = "UPDATE {$this->tab} SET ";
			
		foreach ($data as $key => $val)
		{
			$val = str_replace("'", "&#39;", $val);
			$keyColumn = ':set' . $key;
			$sql .= "`$key`=$keyColumn, ";
			$bindValues[$keyColumn] = $val;
		}

		$sql = rtrim($sql,', ');
		$sql .= ' WHERE '.$where.';';
		$this->_createStatement( $sql, $bindValues )->execute();
		//$this->db->exec($sql);
	}

	/**
	 * @param string $where
	 * @param array $bindValues
	 * @return void
	 */
	public function deleteRows($where, $bindValues = array() )
	{
		if (DB_READ_ONLY_MODE==1) {
			if (!$this->canWrite()) {return $column;}
		}		
		
		$where='WHERE '.$where;
		$sql = "DELETE FROM $this->tab $where";

		if( !empty($bindValues) )
		{
			$this->_createStatement( $sql, $bindValues )->execute();
			return;
		}

		$this->db->exec($sql);
	}

	/**
	 * @param Integer $id
	 * @return void
	 */
	public function delete($id) {
		return $this->deleteRows("id=:id", array(':id' => $id) );
	}

	/**
	 * @param string $where
	 * @param string $order
	 * @param string $what
	 * @param array $bindValues
	 * @return StdClass
	 */
	public function fetchRow($where = null, $order = null, $what = '*', $bindValues = array(), $fetchType = PDO::FETCH_OBJ) {
		$stmt = $this->select($where, $order, $what, $limit = 1, null, $bindValues);
		$stmt->execute();
		return $stmt->fetch( $fetchType );
	}

	/**
	 * @param string $where
	 * @param string $order (default null)
	 * @param string $what (default '*')
	 * @param integer $limit (default null)
	 * @param string $groupby (default null)
	 * @param array $bindValues (default array)
	 * @return array
	 */
	public function fetchAll($where = null, $order = null, $what = '*', $limit = null, $groupby = null, $bindValues = array(),$fetchType = PDO::FETCH_OBJ) {
		$stmt = $this->select($where, $order, $what, $limit, $groupby, $bindValues);
		$stmt->execute();
		return $stmt->fetchAll( $fetchType );
	}

	/**
	 * @param string $searchText
	 * @param string $columns
	 * @param string $where
	 * @param string $order
	 * @param string $what
	 * @param integer $limit
	 * @param string $groupby
	 * @param array $bindValues
	 * @return array
	 */
	public function fetchTextSearch($searchText,array $columns, $where = null, $order = null, $what = '*', $limit = null, $groupby = null, $bindValues = array() ) {
		foreach ($columns as $column){
			$whereFin .= "{$column} LIKE '%{$searchText}%' OR ";
		}
		$whereFin = rtrim($whereFin,' OR ');
		if ($where !=null) {
			$whereFin = " ( ".$whereFin." ) "." AND ( ".$where." )";
		}

		$results = $this->fetchAll($whereFin);
		if(empty($results))return null;

		foreach ($results as $key => $result) {
			$string = null;
			foreach ($columns as $column) {
				$string .= $result->$column;
			};
			$string = strtolower($string);
			$results[$key]->searchElementsCount = substr_count($string, strtolower($searchText));
				

		}
		$results = generator::subvalArrayObjectsSort($results, 'searchElementsCount', 'arsort');
		return $results;
	}

	/**
	 * @param string $where
	 * @param array $bindValues
	 * @return integer
	 */
	public function fetchCount($where = null, $bindValues = array() ) {
		if ($where!=null){$where='WHERE '.$where;}
		$sql = "SELECT count(*) FROM $this->tab $where";

		if( !empty( $bindValues ) )
		{
			$stmt = $this->_createStatement( $sql, $bindValues );
		}
		else
		{
			$stmt = $this->db->query($sql);
		}

		$stmt->execute();
		return $stmt->fetchColumn();
	}

	/**
	 * @param string $groupBy
	 * @param string $where
	 * @param string $order
	 * @param string $what
	 * @param integer $limit
	 * @param array $bindValues
	 * @return array
	 */
	public function fetchCountGroup($groupBy,$where = null,$order = null,$what = '*', $limit = null, $bindValues = array() ) {
		if ($where!=null){$where='WHERE '.$where;}
		if ($order!=null){$order='ORDER BY '.$order;}else{$order="ORDER BY COUNT($groupBy) DESC";}
		if ($limit!=null){$limit='LIMIT '.intval($limit);}

		$sql = "SELECT *, COUNT($groupBy) FROM {$this->tab} $where GROUP BY {$groupBy} $order $limit";

		if( !empty( $bindValues ) )
		{
			$stmt = $this->_createStatement( $sql, $bindValues );
		}
		else
		{
			$stmt = $this->db->query($sql);
		}
		$stmt->execute();

		return $stmt->fetchAll(PDO::FETCH_OBJ);
	}
	/**
	 * @param string $sumColumn
	 * @param string $where
	 * @param string $order
	 * @param string $what
	 * @param integer $limit
	 * @param array $bindValues
	 * @return integer
	 */
	public function fetchSum($sumColumn,$where = null,$order = null,$what = '*', $limit = null, $bindValues = array() ) {
		if ($where!=null){$where='WHERE '.$where;}
		if ($order!=null){$order='ORDER BY '.$order;}
		if ($limit!=null){$limit='LIMIT '.intval($limit);}

		$sql = "SELECT sum({$sumColumn}) AS sum FROM {$this->tab} $where $order $limit";

		if( !empty( $bindValues ) )
		{
			$stmt = $this->_createStatement( $sql, $bindValues );
		}
		else
		{
			$stmt = $this->db->query($sql);
		}
		$stmt->execute();
		return $stmt->fetch(PDO::FETCH_OBJ)->sum;
	}

	/**
	 * @param string $groupBy
	 * @param string $where
	 * @param string $order
	 * @param string $what
	 * @param integer $limit
	 * @param array $bindValues
	 * @return array
	 */
	public function fetchGroup($groupBy, $where = null, $order = null, $what = '*', $limit = null, $bindValues = array()) {
		$groups = $this->select($where, $order, $groupBy, null, $groupBy, $bindValues);
		$groups->execute();
		$groups = $groups->fetchAll( PDO::FETCH_OBJ );

		if ($where!=null){$where=$where.' AND ';}
		foreach ($groups as $group) {
			$rows = $this->select($where."{$groupBy} = '{$group->$groupBy}'", $order, $what, $limit, null, $bindValues);
			$rows->execute();
			$rows = $rows->fetchAll( PDO::FETCH_OBJ );

			if($rows!=null){
				$data[$group->$groupBy] = $rows;
			}
			$rows = null;
		}
		return $data;
	}

	/**
	 * @param integer $id
	 * @param string $column
	 * @param string $value
	 * @return void
	 */
	public function set($id,$column,$value) {
		return $this->update($column,$value,"id=:id", array( ':id' => $id ) );
	}

	/**
	 * @param integer $id
	 * @param string $column
	 * @return string
	 */
	public function get($id,$column) {
		$stmt = $this->select("id=:id", null, $column, 1, null, array( ':id' => $id ) );
		$stmt->execute();
		return $stmt->fetch(PDO::FETCH_OBJ)->$column;
	}
	public function databaseSize() {
		echo $this->db; exit;
		$sql = "SHOW TABLE STATUS FROM {$this->db}";
		return $this->db->query($sql);
	}
        
        /**
         * @param string $sql
         * @return PDOStatement|array can be array of PDOStatement
         */
	public static function executeSql($sql) {
		if (DB_READ_ONLY_MODE==1) {
			if (!$this->canWrite()) {return $column;}
		}
				
		$instance = parent::getInstance();
		$sqlParts = explode(';',$sql);

		if (is_array($sqlParts) && count($sqlParts) > 1 ) {
                    
			foreach ($sqlParts as $sql){
				$returnedData[] = $instance->query($sql);
			}
		}else{
			$returnedData = $instance->query($sql);
		}

		return $returnedData;
	} 
        
        /**
         * 
         * @param string $sql
         * @param array $bindValues
         * @param integer $fetchType (optional)
         * @return array
         */
        public function fetchAllSql( $sql, $bindValues = array(), $fetchType = PDO::FETCH_OBJ )
        {
            $stmt = $this->executeBindedSql($sql, $bindValues);
            $stmt->execute();
            return $stmt->fetchAll( $fetchType );
        }
        
        /**
         * @param string $sql
         * @param array $bindValues (optional)
         * @return PDOStatement
         */
        public function executeBindedSql( $sql, $bindValues = array() )
        {
            return $this->_createStatement($sql, $bindValues);
        }
        
	public static function getRecordsCount($tableName) {
		$table = new $tableName();
		return $table->fetchCount();
	}
	public static function getColumnsCount($tableName) {
		$table = new $tableName();
		return count($table->fetchRow( null, null, '*', array(), $fetchType = PDO::FETCH_ASSOC));
	}
	public static function getTableSize($tableName) {
		$table = new $tableName();
		return $table->fetchCount();
	}
	
	private function canWrite() {
		$tablesArray = explode(',', DB_READ_ONLY_EXCLUDED_TABLES);
		if (in_array($this->tab, $tablesArray)) {
			return true;
		}
		return false;
	}
}

?>
